//
//	opticsxy: compute optical axis/focal plane intersection point
//
//	USAGE:
//		opticsxy
//
//	DESCRIPTION:
//		This task loops over all imaging instruments and computes the
//		optical axis/focal plane intersection point in CHIPCOORD
//		coordinates. This uses from the CCF:
//			- boresight misalignment angles
//			- location of mirror module in SACCOORD
//			- location of CCDs in SACCOORD
//
//		The output may be put in CCF MISCDATA items OPTICS_X/OPTICS_Y.
//
//	CHANGE HISTORY:
//		UL, 2002-02-18: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers, Mon Feb 18 17:21:09 MET 2002
//
//	$Header: /sasbuild/sas_git_repo/cvs_copy/sasdev/sas/packages/ccftools/bstools/src/opticsxy.cc,v 1.5 2002/12/12 16:00:56 ulammers Exp $
//
#include <iostream>
#include <iomanip>

#include "CalServer.h"
#include "Celestial.h"
#include "GeometryDataServer.h"
#include "Xmm.h"

#include "opticsxy.h"

using namespace std;

void opticsxy()
{
	const vector<XmmInstrument *> &ins = xmmPayLoad.instruments();
	for (unsigned i=0; i<ins.size(); ++i) {
		if (!ins[i]->isImaging())
			continue;

		calServer.setInstrument(ins[i]->id());
		calServer.state()->ccd()->set(ins[i]->focusCcd());

		//
		//	Retrieve boresight misalignment matrix (BSM) of set instrument
		//	from CCF
		//
		Boresight *bs = 0; bs = calServer.getAtom(bs);
		AttitudeMatrix BSM = bs->matrix(0.);

		GeometryDataServer *gds = 0; gds = calServer.getAtom(gds);
		const CoordFrameQuantities &cfq= gds->coordFrameQuantities();

		//
		//	compute optical axis / CCD plane intersection point
		//
		//	plane:	N dot (P-P3) == 0
		//	line:	P = P1 + u * optAxis
		//	u of intersection point: u = N dot (P3-P1)/N dot optAxis
		//
		CartesianVector P1 = cfq.MM0,	//	center of mirror module in
										//	SACCOORD
						optAxis = -BSM.xRefAxis(),
										//	unit vector along optical axis
										//	pointing to focal plane
						P3 = cfq.cc2ToSacCoordTrans,
										//	origin of CAMCOORD2 in SACCOORD
						N = cfq.cc2ToSacCoordMatrix.inverse().zRefAxis();
										//	normal on CAMCOORD2 frame;

		double u = N*(P3-P1)/(N*optAxis);
		CartesianVector r = P1+u*optAxis;
		double xmm = r.x(),
			   ymm = r.y(),
			   zmm = r.z();
		CalReal64Vector xv(&xmm, 1), yv(&ymm, 1), zv(&zmm, 1);

		XmmCartesianPointList cpl = gds->sacCoordList(xv, yv, zv);
		cpl.toCamCoord2().toCamCoord1();
		XmmPixelList ccl = cpl.toChipCoord();
		cout<<ins[i]->id()<<':'<<endl;
		cout<<"OPTICS_X="<<ccl.x()[0]<<" - OPTICS_Y="<<ccl.y()[0]
			<<" - OPTICS_CCD="<<ins[i]->focusCcd()<<endl;
	}
}
