//
//	strbs: computes STar Tracker BoreSight from BS of specified instrument
//
//	USAGE:
//		strbs instrument=<instr> ra=<ra> dec=<dec> apos=<apos>
//			  odffixout=yes|no bstoolsout=yes|no
//
//	CHANGE HISTORY:
//		UL, 2000-03-21: 1.0.0: - first version
//		UL, 2000-03-22: 1.1.0: - implemented odffixout parameter
//		UL, 2000-04-04: 1.2.0: - bug fix
//		UL, 2000-05-04: 1.3.0: - more code comments
//							   - bug fix
//		UL, 2000-05-12: 1.3.1: - corrected typo in diagnostic message
//		UL, 2000-07-03: 1.3.2: - give Ra/Dec with now 8 significant digits
//								 in case of odffixoutput=true
//		UL, 2000-09-13: 1.3.2: - added 'bstoolsout' switch to output result
//								 as command line for 'insbs'
//		UL, 2003-02-06: 1.4.0: - ra/dec parameters are now of type angle
//
//	AUTHOR:
//		U. Lammers, Tue Nov 30 14:34:52 MET 1999
//
//	$header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/ccftools/bstools/src/strbs.cc,v 1.10 2000/07/03 14:45:55 ulammers Exp $
//
#include <iostream>
#include <iomanip>

#include "CalServer.h"
#include "Celestial.h"
#include "errstr.h"
#include "param.h"
#include "errstr.h"

#include "strbs.h"

void strbs()
{
	//
	//	read input parameters:
	//		Right Ascension/Declination of instrument boresight + astronomical
	//		position angle
	//
	RightAscension ra(realParameter("ra")*DEG_TO_RAD);
	Declination dec(realParameter("dec")*DEG_TO_RAD);
	double apos = realParameter("apos")*DEG_TO_RAD;

	//
	//	construct instrument attitude matrix from instrumental boresight (INS) 
	//	+ APA with a 3-2-1 Euler angle sequence; Ra of INS is directly
	//	3-angle; Dec of INS + APA have to be negated to be interpreted
	//	as 2-1 angles respectively
	//
	AttitudeMatrix BSA = AttitudeMatrix(ra.angle(), -dec.angle(), -apos,
										EulerAngles::Euler321);
	errstr<<"Instrument attitude matrix:\n"<<BSA<<endl
		  <<Sas::message(Msg::App, Msg::Noisy);


	//
	//	Retrieve boresight misalignment matrix (BSM) of specified instrument
	//	from CCF
	//
	string instrString(stringParameter("instrument"));
	calServer.setInstrument(Xmm::toInstrument(instrString));
	Boresight *bs = 0; bs = calServer.getAtom(bs);
	AttitudeMatrix BST = bs->matrix(0.).inverse();
	errstr<<"Inverse boresight misalignment matrix:\n"<<BST<<endl
		  <<Sas::message(Msg::App, Msg::Noisy);

	//
	//	The BSM maps vectors from the S/C frame into the instrument BS frame
	//	i.e., the INS frame is defined as:
	//		BSA = BSM * ASC
	//	with
	//		BSA	: direction cosine matrix of INS
	//		BSM : boresight misalignment matrix
	//		ASC : direction cosine matrix of S/C frame
	//
	//	hence:
	//		ASC = BST * BSA
	//
	//	with
	//				 -1		 T      
	//		BST = BSM   = BSM
	//
	AttitudeMatrix ASC = BST * BSA;
	errstr<<"Spacecraft attitude matrix:\n"<<ASC<<endl
		  <<Sas::message(Msg::App, Msg::Noisy);


	//
	//	now compute Ra/Dec of star tracker viewing direction which is
	//	-X axis of S/C reference frame; ASC maps vectors from the
	//	reference frame into the body frame, hence
	//	   -1
	//	ASC	  maps vectors from the body frame into the reference frame
	//
	EulerAngles ea = ASC.toEulerAngles(EulerAngles::Euler321);
	CartesianVector strBSv = ASC.inverse() * CartesianVector(-1., 0., 0.);
	errstr<<"Star tracker viewing direction vector: "
		  <<setprecision(12)<<strBSv<<endl<<Sas::message(Msg::App, Msg::Noisy);
	EquatorialDirection strBS(strBSv);

	if (!booleanParameter("odffixout") && !booleanParameter("bstoolsout"))
		cout<<"Viewing direction of STR with "<<instrString<<" prime:\n"
			<<strBS.ra()<<" / "<<strBS.dec()<<" APOS= "
			<<setprecision(5)<<ea.psi()*RAD_TO_DEG<<"deg\n";
	else {
		//	output in odffix format
		double psi = ea.psi();
		if (psi > M_PI)
			psi -= M_TWO_PI;	//	odffix needs -Pi<=apos<=Pi
		cout<<setprecision(8)<<"ra="<<strBS.ra().angle()*RAD_TO_DEG
			<<" dec="<<strBS.dec().angle()*RAD_TO_DEG
			<<(booleanParameter("odffixout") ? " posangle=" : " apos=")
			<<psi*RAD_TO_DEG;
	}
}
